#ifndef R2Vector_H
#define R2Vector_H

#include <iostream>
#include <cmath>
using namespace std;

/* Vector class built for R2 */
class RVector {
 public:
  double x,y;

  RVector(double X,double Y){x=X;y=Y;}
  RVector(){x=y=0.0;}
  ~RVector(){}
  
  // Simple operator overloading
  friend ostream& operator << (ostream &os, const RVector &R) {
      os << R.x << ' ' << R.y << ' ';
      return os; }

  RVector(const RVector &B){x=B.x;y=B.y;}

  RVector& operator=(const RVector& B) {
    x=B.x; y=B.y;
    return *this; }

  RVector operator+ (const RVector &B) {
    return RVector (x+B.x, y+B.y); }
  
  RVector operator- (const RVector &B) {
    return RVector(x-B.x, y-B.y);  }

  RVector& operator+=(const RVector& B) {
    x+=B.x; y+=B.y; return *this;}

  RVector& operator-=(const RVector& B) {
    x-=B.x; y-=B.y; return *this;}

  RVector& operator*=(double c){x*=c; y*=c; return *this;}


  RVector& operator* (double c){ return RVector(*this) *= c; }

  // Some member functions
  double abs() { return sqrt(x*x+y*y); }  
  const RVector unit() { 
    // DEBUG HERE
    if (abs()==0) cout << "DIEEEEEEEEEEEEEEEE!" << endl;
    // DEBUG HERE
    return RVector(x,y)*(1./abs()); 
  }
  double dot(const RVector &B) { return x*B.x + y*B.y; }
};

// Non-member functions that allow pasing const parameters
//double  dot(const RVector &A, const RVector &B) {
//  return  A.x*B.x + A.y*B.y; }

double abs(const RVector &A) { return sqrt(A.x*A.x+A.y*A.y); }
double dot(const RVector &A, const RVector &B) { return A.x*B.x+A.y*B.y; }

RVector  operator*(const double c, const RVector &A) {
  return RVector( A.x*c,A.y*c );}

RVector  operator+(const RVector &A, const RVector &B) {
  return  RVector (A.x+B.x,A.y+B.y); }

RVector  operator-(const RVector &A, const RVector &B) {
  return  RVector (A.x-B.x,A.y-B.y); }

bool  operator==(const RVector &A, const RVector &B) {
  return  (A.x==B.x) && (A.y==B.y); }

bool  operator!=(const RVector &A, const RVector &B) {
  return  !((A.x==B.x) && (A.y==B.y)); }

#endif
