#include "R2Vector.h"
#include <iostream>
#include <cmath>

using namespace std;

double const PI   = 3.14159265;
double const PIO2 = 1.57079633;
double const ROOT_TOL = .0001;

double Edelta_root(const double t, const double a, const double b, 
		   const double r, const double m1, const double m2){

  double C = cos(t), S = sin(t);
  double alpha   = b*b*C*C + a*a*(1 - C*C);
  double alpha32 = pow(alpha,1.5);
  double num1  =  (b*S - m1*a*C - m2)*alpha32 + r*  (a*S - m1*b*C)*alpha;
  double num2  = -(b*C + m1*a*S)     *alpha32 - r*a*(C*b + m1*b*a*S);
  return num1/num2;
  // t += num1/num2;
}

bool Eroot_single(const double a,  const double b, const double r, 
	   const double m1, const double m2,double &phi, double inital_value=0){
  phi = inital_value;
  double delta;
  for(int i=0;i<100;i++) {
    delta = Edelta_root(phi,a,b,r,m1,m2);
    if      (delta>PIO2) {delta=PIO2;}
    else if (delta<-PIO2) {delta=-PIO2;}
    phi  += delta;
    if(abs(delta)<ROOT_TOL) {return 1;}
  }
  return 0;
}

double shift_phi(double x){
  double phi = x-2*PI*(int(x)/int(2*PI));
  while(phi<0) phi+=2*PI;
  return phi;
}

// The idea is that the roots should be sepearted by PI as long as 
// Newtons method does not take to big of a jump - hence the limiter
void Eroots(const double a,  const double b,  const double r, 
            const double m1, const double m2, double &phi1, double &phi2,
	    double inital){
  Eroot_single(a,b,r,m1,m2,phi1,inital);
  Eroot_single(a,b,r,m1,m2,phi2,phi1+PI);
}

void Econtact(const double a, const double b,const double r,double phi, RVector &A){
  double C = cos(phi), S = sin(phi);
  double alpha   = sqrt(b*b*C*C+a*a*S*S);
  A.x = C*(a + r*b/alpha);
  A.y = S*(b + r*a/alpha);
}

const double infinite_time = pow(10,10);

double Eimpact_time(const double a, const double b, const double r, 
		    const RVector &R0, const RVector &V0){
  
  double phi1, phi2;
  RVector hit1, hit2;

  double m1 = V0.y/V0.x;
  double m2 = R0.y - m1*R0.x;

  cout << m1 << ' ' << m2 << endl;

  Eroots(a,b,r,m1,m2,phi1,phi2,0);
  Econtact(a,b,r,phi1,hit1);
  Econtact(a,b,r,phi2,hit2);
  
  double t1 = (hit1.x - R0.x)/V0.x;
  double t2 = (hit2.x - R0.x)/V0.x;

  if(t1<0 && t2<0) return infinite_time;
  if(t2<t1) return t2;
  return t1;
}

/*
void handle_pe_collision(  ... ) {
  
  // Determine which spot hit first - set as hit1

  cout << t1 << ' ' << t2 << endl;
  exit(3);
  if(t2<t1) {hit1 = hit2; t1=t2; phi1=phi2;}

  // Handle the collision, T is tangent vector
  double Vmag   = abs(V0);
  double alpha  = sqrt(pow((b*cos(phi1)),2.0) + pow((a*sin(phi1)),2.0));
  RVector T_hat(-a*sin(phi1)/alpha, b*cos(phi1)/alpha);
  RVector Par_hat(1,m1);

  // Find the rotation angle
  double delta = -2*acos( T_hat.dot(Par_hat.unit())) ;
  double C = cos(delta), S = sin(delta);

  // Rotate the velocity vector by this angle
  RVector Vf( C*V0.x - S*V0.y, S*V0.x + C*V0.y );
  return Vf;
  //cout << Vf << endl;
  //cout << abs(Vf) << ' ' << abs(V0) << endl;
}
*/

int main() {
  double a  = 2.0,  b =1.0;
  double m1 =-.25, m2=-1.5;
  double r  = .75;

  RVector R0(-6,0);
  RVector V0(-4,1);

  cout << V0 << endl;
  V0 = Eimpact(a,b,r,R0,V0);
  cout << V0 << endl;


  return 0;
}

