#ifndef ellipise_particle_H
#define ellipse_particle_H
#include <cstdlib>
#include <vector>
#include <string>
#include <vector>
#include <iostream>
#include <cmath>
#include "sphere_particle.h"
#include "R2Vector.h"

class sphere_particle;

double const E_PI   = 3.14159265;
double const E_PIO2 = 1.57079633;
double const ROOT_TOL = 1E-7;
double const E_infinite_time = 10E20;   

double Edelta_root(const double phi, const double a, const double b, 
		   const double r, const double m1, const double m2){

  double C = cos(phi), S = sin(phi);
  double alpha   = b*b*C*C + a*a*(1 - C*C);
  double alpha32 = pow(alpha,1.5);
  double num1  =  (b*S - m1*a*C - m2)*alpha32 + r*  (a*S - m1*b*C)*alpha;
  double num2  = -(b*C + m1*a*S)     *alpha32 - r*a*(C*b + m1*b*a*S);
  return num1/num2;
}

double shift_phi(double x){
  double phi = x-2*E_PI*(int(x)/int(2*E_PI));
  while(phi<0) phi+=2*E_PI;
  return phi;
}

bool Eroot_single(const double a,  const double b, const double r, 
	   const double m1, const double m2,double &phi, double inital_value=0){
  phi = inital_value;
  double delta;
  for(int i=0;i<200;i++) {
    delta = Edelta_root(phi,a,b,r,m1,m2);
    if      (delta>E_PIO2)  {delta=E_PIO2;}
    else if (delta<-E_PIO2) {delta=-E_PIO2;}
    phi  += delta;
    if(abs(delta)<ROOT_TOL) {return 1; }
  }
  return 0;
}

// The idea is that the roots should be sepearted by PI as long as 
// Newtons method does not take to big of a jump - hence the limiter
bool Eroots(const double a,  const double b,  const double r, 
            const double m1, const double m2, double &phi1, double &phi2,
	    double inital){
  bool test = Eroot_single(a,b,r,m1,m2,phi1,inital);
  if(!test) return 0;
  return Eroot_single(a,b,r,m1,m2,phi2,phi1+E_PI);
}

RVector Econtact(const double a, const double b,const double r,double phi){
  double C = cos(phi), S = sin(phi);
  double alpha   = sqrt(b*b*C*C+a*a*S*S);

  RVector A;
  A.x = C*(a + r*b/alpha);
  A.y = S*(b + r*a/alpha);
  return A;
}



#endif


