#include "R2Vector.h"
#include "sphere_particle.h"
#include <iostream>
#include <cmath>
using namespace std;

int main(int argc, char *argv[]) {
  set_windows2D();

  HM_N      = 100;
  HM_radius = Pr;
  HM_max    = Pr*3;
  HM_radius_epsilon = Pr/100.;

  // Histogram paramets
  int bins_hist = 100;
  bins_xspan    = .5;

  // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  // Command-line input parameters
  if (argc==1) {}
  else if (argc<8) {
    cout << "Command line usage:" << endl
	 << " idealgas_collision_prediction [radius]"
	 << " [heatmap radius] [HM radius tolerence] [total_time] [dt] [crowder #] "
	 << " optional: [rand_seed] [bins] [bins_xspan] [HM_N] [HM_max]" << endl;
    return 1;
  }
  else {
    Pr = convertToDouble(argv[1]);
    HM_radius = convertToDouble(argv[2]);
    HM_radius_epsilon = convertToDouble(argv[3]);
    total_time = convertToDouble(argv[4]);
    dt = convertToDouble(argv[5]);
    NP = convertToInt(argv[6]);
  }

  if (argc>7) random_seed = convertToInt(argv[7]);
  if (argc>8) bins_hist   = convertToInt(argv[8]); 
  if (argc>9)bins_xspan  = convertToDouble(argv[9]);
  if (argc>10)HM_N        = convertToInt(argv[10]);
  if (argc>11)HM_max      = convertToDouble(argv[11]);

  // -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  srand(random_seed);

  // Clear out the histogram
  HM_deltar = (HM_max)/float(HM_N);

  C12.resize( HM_N ); 

  for(int i=0;i<HM_N;i++){ C12[i]=0;  }
  
  // Create partices non-overlaping pos. and gaussian  random vel.
  // Avoid the ellipise!
  next_data_t = dt;

  double delta = Pr*2.1;
  double xpos=-L + delta, ypos=-L + delta;
  for(int i=0;i<NP;i++) {
    RVector R(xpos, ypos);
    RVector V(0, 0);
      
    sphere_particle s(R,V,Pr,Pm, P.size());
    set_randominital2D_V(s, .2);
    P.push_back(s);
    xpos += delta;
    
    double g=1+1.5*Pr;
    while(xpos>-g*Ea && xpos<g*Ea && ypos>-g*Eb && ypos<g*Eb){
      xpos += .01; }

    if(xpos>L-delta) {xpos= -L + delta; ypos += delta;}
  }
  
  // Calculate all the inital collisions
  for(int i=0;i<P.size();i++) {calc_Ecollision(P[i]);}
  for(int i=0;i<P.size();i++) {calc_collision(P[i],P);  }


  // Run the simulation
  while(GLOBAL_T < total_time) {
    run_simulation(P);
  }

  // Show the histogram output if wanted
  /*
  cout << NP << ' ' << HM_radius << ' '
       << HM_radius_epsilon << ' ' << dt << ' ' << bins_hist << ' '
       << bins_xspan << ' ' << HM_N << ' ' << HM_max << endl;

  for(int i=0;i<HM_N;i++){ 
    cout << i << ' ' << C12[i] << endl;
    }*/

  return 0;
}

